/*
*  LINUX NETWORK CONNECTION PART for ed2k_slist
*
* --- COPYRIGHT AND LICENCE ---
*
* Copyright (C) 2001 Tim-Philipp Mller (t.i.m@orange.net)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*
*/

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

/* connect with timeout
*
* probably by Mark Pulford <mark@kyne.com.au>
* found on the LinuxSA mailing list <linuxsa@linuxsa.org.au>
* on Wed, 2 Sep 1998 16:48:06 +0930 (CST)
* Subject: "Re: Socket timeout question"
*
*/

int do_connect(int fd,struct sockaddr *remote, int len,
               struct timeval *timeout, int *err)
{
        int saveflags,ret,back_err;
        fd_set fd_w;

        saveflags=fcntl(fd,F_GETFL,0);
        if(saveflags<0) {
                perror("fcntl1");
                *err=errno;
                return -1;
        }

        /* Set non blocking */
        if(fcntl(fd,F_SETFL,saveflags|O_NONBLOCK)<0) {
                perror("fcntl2");
                *err=errno;
                return -1;
        }

        /* This will return immediately */
        *err=connect(fd,remote,len);
        back_err=errno;

        /* restore flags */
        if(fcntl(fd,F_SETFL,saveflags)<0) {
                perror("fcntl3");
                *err=errno;
                return -1;
        }

        /* return unless the connection was successful or the connect is
           still in progress. */
        if(*err<0 && back_err!=EINPROGRESS) {
                perror("connect");
                *err=errno;
                return -1;
        }

        FD_ZERO(&fd_w);
        FD_SET(fd,&fd_w);

        *err=select(FD_SETSIZE,NULL,&fd_w,NULL,timeout);
        if(*err<0) {
                perror("select");
                *err=errno;
                return -1;
        }

        /* 0 means it timeout out & no fds changed */
        if(*err==0) {
                close(fd);
                *err=ETIMEDOUT;
                return -1;
        }

        /* Get the return code from the connect */
        len=sizeof(ret);
        *err=getsockopt(fd,SOL_SOCKET,SO_ERROR,&ret,&len);
        if(*err<0) {
                perror("getsockopt");
                *err=errno;
                return -1;
        }

        /* ret=0 means success, otherwise it contains the errno */
        if (ret!=0) {
                *err=ret;
                return -1;
        }

        *err=0;
        return 0;
}

/* does_server_connect (linux)
*
* returns USEFUL if server connects, otherwise NOT_USEFUL
*
*/

int does_server_connect (donkeyserver *dfp)
{
	int    socket_descriptor;	             /* file descriptor for socket        */
	struct sockaddr_in pin;
	struct hostent *server_host_name;
    struct timeval timeout;
    int    connect_err;

	if (verbose==1)
  		fprintf (stderr, "TRYING %s:%d (%s)... ", dfp->ip_str, dfp->port, dfp->name);

	if ((server_host_name = gethostbyname(dfp->ip_str)) == 0) {
  		if (verbose==1)
    			perror("Error resolving local host");
		return (NOT_USEFUL);
	}

	memset (&pin, 0x00, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
	pin.sin_port = htons(dfp->port);

	if ((socket_descriptor = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if (verbose==1)
			perror("Error opening socket");
        	close (socket_descriptor);
		return (NOT_USEFUL);
	}

        /* set our timeout value */

        timeout.tv_sec  = SERVER_TIMEOUT;    /* seconds                            */
        timeout.tv_usec = 0;     /* micro-seconds (1 us = 1/1000 ms = 1/1000000 s) */

	if (do_connect(socket_descriptor, (void *)&pin, sizeof(pin), &timeout, &connect_err) == -1) {
		if (verbose==1)
			fprintf (stderr,"timed out or connection refused\n");
                close (socket_descriptor);
		return (NOT_USEFUL);
	}

	/* connection must have been successful */

	if (verbose==1)
		fprintf (stderr,"successfully connected!\n");

	close (socket_descriptor);

	return (USEFUL);;
}


